@@ -3,10 +3,10 @@ import numpy as np
 import tensorflow as tf
 import pandas as pd
 import os
-from typing import Union, List
+from typing import Union, List, Callable
 from collections.abc import MutableMapping
 from kgcnn.data.utils import save_pickle_file, load_pickle_file, ragged_tensor_from_nested_numpy
-from kgcnn.graph.base import GraphNumpyContainer, GraphDict
+from kgcnn.graph.base import GraphDict
 
 logging.basicConfig()  # Module logger
 module_logger = logging.getLogger(__name__)
@@ -204,13 +204,14 @@ class MemoryGraphList(MutableMapping):
         else:
             raise TypeError("Wrong type, expected e.g. [{'name': 'edge_indices', 'ragged': True}, {...}, ...]")
 
-    def map_list(self, method: str, **kwargs):
+    def map_list(self, method: Union[str, Callable], **kwargs):
         r"""Map a method over this list and apply on each :obj:`GraphDict`.
+        For :obj:`method` being string, either a class-method or a preprocessor is chosen for backward compatibility.
 
         .. code-block:: python
 
             for i, x in enumerate(self):
-                getattr(x, method)(**kwargs)
+                method(x, **kwargs)
 
         Args:
             method (str): Name of the :obj:`GraphDict` method.
@@ -219,9 +220,22 @@ class MemoryGraphList(MutableMapping):
         Returns:
             self
         """
-        for i, x in enumerate(self._list):
-            # Can add progress info here.
-            getattr(x, method)(**kwargs)
+        # Can add progress info here.
+        # Method by name.
+        if isinstance(method, str):
+            for i, x in enumerate(self._list):
+                # If this is a class method.
+                if hasattr(x, method):
+                    getattr(x, method)(**kwargs)
+                else:
+                    # For compatibility names can refer to preprocessors.
+                    x.apply_preprocessor(name=method, **kwargs)
+        elif isinstance(method, dict):
+            raise NotImplementedError("Serialization for method in `map_list` is not yet supported")
+        else:
+            # For any callable method to map.
+            for i, x in enumerate(self._list):
+                method(x, **kwargs)
         return self
 
     def clean(self, inputs: Union[list, str]):
@@ -321,48 +335,52 @@ class MemoryGraphDataset(MemoryGraphList):
         # For logging.
         self.logger = logging.getLogger("kgcnn.data." + dataset_name) if dataset_name is not None else module_logger
         self.logger.setLevel(verbose)
-        # Dataset information on file.
+        # Information on location of dataset on file. Data directory must be filepath.
         self.data_directory = data_directory
         self.file_name = file_name
         self.file_directory = file_directory
         self.dataset_name = dataset_name
-        # Data Frame for information.
+        # Data Frame for labels and graph names.
         self.data_frame = None
         self.data_keys = None
         self.data_unit = None
 
     @property
     def file_path(self):
+        r"""Construct filepath from 'file_name' given in `init`."""
         if self.data_directory is None:
             self.warning("Data directory is not set.")
             return None
         if not os.path.exists(os.path.realpath(self.data_directory)):
             self.error("Data directory does not exist.")
         if self.file_name is None:
-            self.warning("Can not determine file path.")
+            self.warning("Can not determine file path, missing `file_name`.")
             return None
         return os.path.join(self.data_directory, self.file_name)
 
     @property
     def file_directory_path(self):
-        """Returns path information of `file_directory_`."""
+        r"""Construct file-directory path from 'data_directory' and 'file_directory' given in `init`."""
         if self.data_directory is None:
             self.warning("Data directory is not set.")
             return None
         if not os.path.exists(self.data_directory):
             self.error("Data directory does not exist.")
         if self.file_directory is None:
-            self.warning("Can not determine file directory.")
+            self.warning("Can not determine file directory, missing `file_directory`.")
             return None
         return os.path.join(self.data_directory, self.file_directory)
 
     def info(self, *args, **kwargs):
+        """Pass information to class' logger instance."""
         self.logger.info(*args, **kwargs)
 
     def warning(self, *args, **kwargs):
+        """Pass warning to class' logger instance."""
         self.logger.warning(*args, **kwargs)
 
     def error(self, *args, **kwargs):
+        """Pass error to class' logger instance."""
         self.logger.error(*args, **kwargs)
 
     def save(self, filepath: str = None):
@@ -395,7 +413,7 @@ class MemoryGraphDataset(MemoryGraphList):
     def read_in_table_file(self, file_path: str = None, **kwargs):
         r"""Read a data frame in :obj:`data_frame` from file path. By default, uses :obj:`file_name` and pandas.
         Checks for a '.csv' file and then for Excel file endings. Meaning the file extension of file_path is ignored
-        but must be any of the following '.csv', '.xls', '.xlsx', 'odt'.
+        but must be any of the following '.csv', '.xls', '.xlsx', '.odt'.
 
         Args:
             file_path (str): File path to table file. Default is None.
@@ -406,8 +424,10 @@ class MemoryGraphDataset(MemoryGraphList):
         """
         if file_path is None:
             file_path = os.path.join(self.data_directory, self.file_name)
-        file_path_base = os.path.splitext(file_path)[0]
+
+        # TODO: Better determine file-type from file ending and just test all supported types otherwise.
         # file_extension_given = os.path.splitext(file_path)[1]
+        file_path_base = os.path.splitext(file_path)[0]
 
         for file_extension in [".csv"]:
             if os.path.exists(file_path_base + file_extension):
@@ -421,9 +441,12 @@ class MemoryGraphDataset(MemoryGraphList):
         self.warning("Unsupported data extension of '%s' for table file." % file_path)
         return self
 
-    def assert_valid_model_input(self, hyper_input: list, raise_error_on_fail: bool = True):
-        r"""Interface to hyperparameter. Check whether dataset has graph-properties (tensor format) requested
-        by model input. The model input is set up by a list of layer configs for the keras :obj:`Input` layer.
+    def assert_valid_model_input(self, hyper_input: Union[list, dict], raise_error_on_fail: bool = True):
+        r"""Check whether dataset has graph properties (in tensor format) requested by model input.
+
+        The list :obj:`hyper_input` that defines model input match interface to hyperparameter.
+        The model input is set up by a list of layer configs for the keras :obj:`Input` layer.
+        The list must contain dictionaries for each model input with "name" and "shape" keys.
 
         .. code-block:: python
 
@@ -448,6 +471,21 @@ class MemoryGraphDataset(MemoryGraphList):
         def message_warning(msg):
             dataset.warning(msg)
 
+        if isinstance(hyper_input, dict):
+            if "name" in hyper_input and "shape" in hyper_input:
+                # Single model input that has not been properly passed as list.
+                # Assume here one does not name model output name and shape.
+                hyper_input = [hyper_input]
+            else:
+                # In principle keras also accepts a dictionary for model inputs. Just cast to list here.
+                hyper_input = list(hyper_input.values())
+
+        # Check if we have List[dict].
+        for x in hyper_input:
+            if not isinstance(x, dict):
+                message_error(
+                    "Wrong type of list item in `assert_valid_model_input`. Found %s but must be `dict`" % type(x))
+
         for x in hyper_input:
             if "name" not in x:
                 message_error("Can not infer name from '%s' for model input." % x)
@@ -474,6 +512,60 @@ class MemoryGraphDataset(MemoryGraphList):
                 message_error("Can not check shape for '%s'." % x["name"])
         return
 
+    def collect_files_in_file_directory(self, file_column_name: str = None, table_file_path: str = None,
+                                        read_method_file: Callable = None, update_counter: int = 1000,
+                                        append_file_content: bool = True,
+                                        read_method_return_list: bool = False) -> list:
+        r"""Utility function to collect single files in :obj:`file_directory` by names in CSV table file.
+
+        Args:
+            file_column_name (str): Name of the column in Table file that holds list of file names.
+            table_file_path (str): Path to table file. Can be None. Default is None.
+            read_method_file (Callable): Callable read-file method to return (processed) file content.
+            update_counter (int): Loop counter to show progress. Default is 1000.
+            append_file_content (bool): Whether to append or add return of :obj:`read_method_file`.
+            read_method_return_list (bool): Whether :obj:`read_method_file` returns list of items or the item itself.
+
+        Returns:
+            list: File content loaded from single files.
+        """
+        self.read_in_table_file(table_file_path)
+        if self.data_frame is None:
+            raise FileNotFoundError("Can not find '.csv' table path '%s'." % table_file_path)
+
+        if file_column_name is None:
+            raise ValueError("Please specify column for '.csv' file which contains file names.")
+
+        if file_column_name not in self.data_frame.columns:
+            raise ValueError(
+                "Can not find file names of column '%s' in '%s'" % (file_column_name, self.data_frame.columns))
+
+        name_file_list = self.data_frame[file_column_name].values
+        num_files = len(name_file_list)
+
+        if not os.path.exists(self.file_directory_path):
+            raise ValueError("No file directory found at '%s'." % self.file_directory_path)
+
+        self.info("Read %s single files." % num_files)
+        out_list = []
+        for i, x in enumerate(name_file_list):
+            # Only one file per path
+            file_loaded = read_method_file(os.path.join(self.file_directory_path, x))
+            if append_file_content:
+                if read_method_return_list:
+                    out_list.append(file_loaded[0])
+                else:
+                    out_list.append(file_loaded)
+            else:
+                if read_method_return_list:
+                    out_list += file_loaded
+                else:
+                    out_list += [file_loaded]
+            if i % update_counter == 0:
+                self.info("... Read {0} file {1} from {2}".format(os.path.splitext(x)[1], i, num_files))
+
+        return out_list
+
     def set_methods(self, method_list: List[dict]) -> None:
         r"""Apply a list of serialized class-methods on the dataset.
 
@@ -500,5 +592,90 @@ class MemoryGraphDataset(MemoryGraphList):
                 else:
                     self.error("Class does not have method '%s'." % method)
 
+    def get_split_indices(self, name: str = "kfold", return_as_train_test: bool = True,
+                          shuffle: bool = True, seed: int = None):
+        """Gather split ids from split graph property and return k-fold splits.
+
+        Args:
+            name (str): Name of property containing split assignment. Default is "kfold".
+            return_as_train_test (bool): Whether to return the splits as train test indices. Default is True.
+            shuffle (bool): Whether to shuffle splits. Default is True.
+            seed (int): Random seed for shuffle. Default is None.
+
+        Returns:
+            list: List of splits.
+        """
+        split_indices = [[]]
+
+        def check_and_extend_splits(to_split):
+            if to_split - len(split_indices) + 1 > 0:
+                for _ in range(to_split - len(split_indices) + 1):
+                    split_indices.append([])
+
+        graphs = self.obtain_property(name)
+        for i, s in enumerate(graphs):
+            if s is None:
+                self.error("Split assignment on graph '%s' is not defined." % i)
+                continue
+            for x in s:
+                check_and_extend_splits(x)
+                split_indices[int(x)].append(i)
+
+        split_indices = [np.array(x, dtype="int") for x in split_indices]
+
+        if shuffle:
+            np.random.seed(seed)
+            for x in split_indices:
+                np.random.shuffle(x)
+
+        if not return_as_train_test:
+            return split_indices
+
+        train_test = []
+
+        for i in range(len(split_indices)):
+            train = np.concatenate([x for j, x in enumerate(split_indices) if j != i], axis=0)
+            test = split_indices[i]
+            train_test.append([train, test])
+
+        return train_test
+
+    def get_train_test_indices(self, train: str = "train", test: str = "test", valid: str = None,
+                               split_index: Union[int, list] = 1):
+        """Get train and test indices from graph list. The 'train' and 'test' properties must be set on the graph.
+        They can also be a list of split assignment if more than one train-test split is required.
+
+        Args:
+            train (str): Name of graph property that has train split assignment. Defaults to 'train'.
+            test (str): Name of graph property that has test split assignment. Defaults to 'test'.
+            valid (str): Name of graph property that has validation assignment. Defaults to None.
+            split_index (int, list): Split index to get indices for. Can also be list.
+
+        Returns:
+            list: list of train, test, validation split indices.
+        """
+        out_indices = []
+        if not isinstance(split_index, (list, tuple)):
+            split_index = [split_index]
+        for s in [train, test, valid]:
+            if s is None:
+                out_indices.append(None)
+                continue
+            s_list = []
+            split_prop = self.obtain_property(s)
+            for j in split_index:
+                split_list = []
+                for i, x in enumerate(split_prop):
+                    if x is not None:
+                        if j in x:
+                            split_list.append(i)
+                s_list.append(split_list)
+            if len(s_list) == 1:
+                out_indices.append(s_list[0])
+            else:
+                out_indices.append(s_list[0])
+
+        return out_indices
+
 
 MemoryGeometricGraphDataset = MemoryGraphDataset
