@@ -152,6 +152,13 @@ def add_self_loops_to_edge_indices(edge_indices, *args,
         np.ndarray: `edge_indices` or `(edge_indices, *args)`. Or `(edge_indices, args)` if `return_nested`.
     """
     clean_edge = [x for x in args]
+    if len(edge_indices) <= 0:
+        if return_nested:
+            return edge_indices, clean_edge
+        if len(clean_edge) > 0:
+            return [edge_indices] + clean_edge
+        else:
+            return edge_indices
     max_ind = np.max(edge_indices)
     self_loops = np.arange(max_ind + 1, dtype="int")
     self_loops = np.concatenate([np.expand_dims(self_loops, axis=-1), np.expand_dims(self_loops, axis=-1)], axis=-1)
@@ -290,68 +297,92 @@ def make_adjacency_from_edge_indices(edge_indices, edge_values=None, shape=None)
     return out_adj
 
 
-def get_angle_indices(idx, check_sorted: bool = True, allow_multi_edges: bool = False):
+def get_angle_indices(idx, check_sorted: bool = True, allow_multi_edges: bool = False,
+                      allow_self_edges: bool = False, allow_reverse_edges: bool = False,
+                      edge_pairing: str = "jk"):
     r"""Compute index list for edge-pairs forming an angle. Not for batches, only for single instance.
 
     Args:
         idx (np.ndarray): List of edge indices referring to nodes of shape `(N, 2)`
         check_sorted (bool): Whether to sort for new angle indices. Default is True.
-        allow_multi_edges (bool): Whether to keep angle pairs with :math:`i=k` in the tuple :math:`(i, j, k)`
+        allow_self_edges (bool): Whether to allow the exact same edge in an angle pairing. Overrides multi and reverse
+            edge checking.
+        allow_multi_edges (bool): Whether to keep angle pairs with same node indices,
+            such as angle pairings of sort `ij`, `ij`.
+        allow_reverse_edges (bool): Whether to keep angle pairs with reverse node indices,
+            such as angle pairings of sort `ij`, `ji`.
+        edge_pairing (str): Determines which edge pairs for angle computation are chosen. Default is 'jk'.
+            Alternatives are for example: 'ik', 'kj', 'ki', where 'k' denotes the variable index as 'i', 'j' are fixed.
 
     Returns:
         tuple: idx, idx_ijk, idx_ijk_ij
 
-        - idx (np.ndarray): Edge indices referring to nodes of shape `(N, 2)`.
-        - idx_ijk (np.ndarray): Indices of nodes forming an angle as i<-j<-k of shape `(M, 3)`.
-        - idx_ijk_ij (np.ndarray): Indices for angle pairs referring to edges of shape `(M, 2)`.
+        - idx (np.ndarray): Original edge indices referring to nodes of shape `(N, 2)`.
+        - idx_ijk (np.ndarray): Indices of nodes forming an angle as (i ,j, k) of shape `(M, 3)`.
+        - idx_ij_jk (np.ndarray): Indices for edge pairs referring to angles of shape `(M, 2)`.
     """
     if idx is None:
         return None, None, None
     if len(idx) == 0:
         return np.array([]), np.array([]), np.array([])
-    # Can swap here
-    idx_i = idx[:, 0]
-    idx_j = idx[:, 1]
-    label_ij = np.arange(len(idx))
-    # Direction of edges is i<-j here, so angles around i<-j<-k
-    idx_ijk = []  # index triples that form an angle as i<-j<-k
-    idx_ijk_ij = []  # New indices that refer to edges to form an angle as i',j'   i':(i, j) j':(j, k)
-    max_j = np.amax(idx_j)
-    for n in range(max_j + 1):
-        mask_jn = idx_j == n
-        mask_in = idx_i == n
-        pos_i = idx_i[mask_jn]
-        pos_j = idx_j[mask_jn]
-        pos_k = idx_j[mask_in]
-        label_i = label_ij[mask_jn]
-        label_k = label_ij[mask_in]
-        if len(pos_i) == 0 or len(pos_j) == 0 or len(pos_k) == 0:
-            continue
-        # All combos around j
-        combos_ik = np.concatenate(
-            [np.expand_dims(np.repeat(np.expand_dims(pos_i, axis=1), len(pos_k), axis=1), axis=-1),
-             np.expand_dims(np.repeat(np.expand_dims(pos_j, axis=1), len(pos_k), axis=1), axis=-1),
-             np.expand_dims(np.repeat(np.expand_dims(pos_k, axis=0), len(pos_i), axis=0), axis=-1)], axis=-1)
-        combos_label = np.concatenate(
-            [np.expand_dims(np.repeat(np.expand_dims(label_i, axis=1), len(label_k), axis=1), axis=-1),
-             np.expand_dims(np.repeat(np.expand_dims(label_k, axis=0), len(label_i), axis=0), axis=-1)], axis=-1)
+    # Labeling edges.
+    label_ij = np.expand_dims(np.arange(len(idx)), axis=-1)
+    # Find edge pairing indices.
+    if "k" not in edge_pairing:
+        raise ValueError("Edge pairing must have index 'k'.")
+    if "i" not in edge_pairing and "j" not in edge_pairing:
+        raise ValueError("Edge pairing must have at least one fix index 'i' or 'j'.")
+    pos_k = 0 if edge_pairing[0] == "k" else 1
+    pos_fix = 0 if edge_pairing[0] != "k" else 1
+    pos_ij = 0 if "i" in edge_pairing else 1
+
+    idx_ijk = []  # index triples that form an angle as (i, j, k)
+    idx_ij_k = []  # New indices that refer to edges to form an angle as ij, `edge_pairing`
+    for n, ij in enumerate(idx):
+        matching_edges = idx
+        matching_labels = label_ij
+
+        # Condition to find matching xk or kx.
+        mask = matching_edges[:, pos_fix] == ij[pos_ij]
 
-        mask_combos = np.ones(combos_label.shape[:2], dtype="bool")
         if not allow_multi_edges:
-            mask_combos = np.array(combos_ik[:, :, 0] != combos_ik[:, :, -1], dtype="bool")
-        idx_ijk.append(combos_ik[mask_combos])
-        idx_ijk_ij.append(combos_label[mask_combos])
-    idx_ijk = np.concatenate(idx_ijk, axis=0)
-    idx_ijk_ij = np.concatenate(idx_ijk_ij, axis=0)
-    if check_sorted:
-        order1 = np.argsort(idx_ijk_ij[:, 1], axis=0, kind='mergesort')  # stable!
-        idx_ijk_ij = idx_ijk_ij[order1]
-        idx_ijk = idx_ijk[order1]
-        order2 = np.argsort(idx_ijk_ij[:, 0], axis=0, kind='mergesort')
-        idx_ijk = idx_ijk[order2]
-        idx_ijk_ij = idx_ijk_ij[order2]
+            mask = np.logical_and(mask, np.logical_or(matching_edges[:, 0] != ij[0], matching_edges[:, 1] != ij[1]))
 
-    return idx, idx_ijk, idx_ijk_ij
+        if not allow_reverse_edges:
+            mask = np.logical_and(mask, np.logical_or(matching_edges[:, 0] != ij[1], matching_edges[:, 1] != ij[0]))
+
+        if allow_self_edges:
+            mask[n] = True
+        else:
+            mask[n] = False
+
+        matching_edges, matching_labels = matching_edges[mask], matching_labels[mask]  # apply mask
+
+        if len(matching_edges) == 0:
+            idx_ijk.append(np.empty((0, 3), dtype=idx.dtype))
+            idx_ij_k.append(np.empty((0, 2), dtype=idx.dtype))
+            continue
+
+        # All combos for edge ij
+        combos_ik = np.concatenate(
+            [np.repeat([ij], len(matching_edges), axis=0), np.expand_dims(matching_edges[:, pos_k], axis=-1)], axis=-1)
+        combos_label = np.concatenate(
+            [np.repeat([[n]], len(matching_labels), axis=0), matching_labels], axis=-1)
+        idx_ijk.append(combos_ik)
+        idx_ij_k.append(combos_label)
+
+    idx_ijk = np.concatenate(idx_ijk, axis=0)
+    idx_ij_k = np.concatenate(idx_ij_k, axis=0)
+
+    if check_sorted:
+        order1 = np.argsort(idx_ij_k[:, 1], axis=0, kind='mergesort')  # stable!
+        idx_ij_k = idx_ij_k[order1]
+        idx_ijk = idx_ijk[order1]
+        order2 = np.argsort(idx_ij_k[:, 0], axis=0, kind='mergesort')
+        idx_ijk = idx_ijk[order2]
+        idx_ij_k = idx_ij_k[order2]
+
+    return idx, idx_ijk, idx_ij_k
 
 
 def get_angle(coord, indices):
@@ -473,9 +504,11 @@ def invert_distance(d, nan=0, pos_inf=0, neg_inf=0):
     return c
 
 
-def distance_to_gauss_basis(inputs, bins: int = 20, distance: float = 4.0, sigma: float = 0.4, offset: float = 0.0):
+def distance_to_gauss_basis(inputs, bins: int = 20, distance: float = 4.0, sigma: float = 0.4, offset: float = 0.0,
+                            axis: int = -1, expand_dims: bool = True):
     r"""Convert distance array to smooth one-hot representation using Gaussian functions.
-    Changes shape for Gaussian distance expansion from `(..., )` to (..., #bins).
+    Changes shape for Gaussian distance expansion from `(..., )` to (..., bins) by default.
+
     Note: The default values match realistic units in Angstrom for atoms or molecules.
 
     Args:
@@ -484,19 +517,21 @@ def distance_to_gauss_basis(inputs, bins: int = 20, distance: float = 4.0, sigma
         distance (value): Maximum distance to be captured by bins. Default is 4.0.
         sigma (value): Sigma of the Gaussian function, determining the width/sharpness. Default is 0.4.
         offset (float): Possible offset to center Gaussian. Default is 0.0.
+        axis (int): Axis to expand distance. Defaults to -1.
+        expand_dims (bool): Whether to expand dims. Default to True.
 
     Returns:
         np.ndarray: Array of Gaussian distance with expanded last axis `(..., #bins)`
     """
     gamma = 1 / sigma / sigma * (-1) / 2
-    d_shape = inputs.shape
-    edge_dist_grid = np.expand_dims(inputs, axis=-1)
-    edge_gauss_bin = np.arange(0, bins, 1) / bins * distance
-    edge_gauss_bin = np.broadcast_to(edge_gauss_bin, np.append(np.ones(len(d_shape), dtype=np.int32),
-                                                               edge_gauss_bin.shape))  # shape (1,1,...,GBins)
-    edge_gauss_bin = np.square(edge_dist_grid - edge_gauss_bin - offset) * gamma  # (N,M,...,1) - (1,1,...,GBins)
-    edge_gauss_bin = np.exp(edge_gauss_bin)
-    return edge_gauss_bin
+    if expand_dims:
+        inputs = np.expand_dims(inputs, axis=axis)
+    gauss_bins = np.arange(0, bins, 1) / bins * distance
+    expanded_shape = [1]*len(inputs.shape)
+    expanded_shape[axis] = len(gauss_bins)
+    gauss_bins = np.broadcast_to(gauss_bins, expanded_shape)
+    output = np.square(inputs - gauss_bins - offset) * gamma  # (N,M,...,1) - (1,1,...,GBins)
+    return np.exp(output)
 
 
 def define_adjacency_from_distance(distance_matrix, max_distance=np.inf, max_neighbours=np.inf, exclusive=True,
