@@ -1,10 +1,11 @@
 import numpy as np
 import logging
 from typing import Union
-from kgcnn.graph.base import GraphPreProcessorBase, GraphDict
+from kgcnn.graph.base import GraphPreProcessorBase
 from kgcnn.graph.adj import get_angle_indices, coordinates_to_distancematrix, invert_distance, \
     define_adjacency_from_distance, sort_edge_indices, get_angle, add_edges_reverse_indices, \
-    rescale_edge_weights_degree_sym, add_self_loops_to_edge_indices, compute_reverse_edges_index_map
+    rescale_edge_weights_degree_sym, add_self_loops_to_edge_indices, compute_reverse_edges_index_map, \
+    distance_to_gauss_basis
 from kgcnn.graph.geom import range_neighbour_lattice
 
 logging.basicConfig()  # Module logger
@@ -25,7 +26,8 @@ class MakeUndirectedEdges(GraphPreProcessorBase):
         remove_duplicates (bool): Whether to remove duplicates within the new edges. Default is True.
         sort_indices (bool): Sort indices after adding edges. Default is True.
     """
-    def __init__(self, edge_indices: str = "edge_indices", edge_attributes: str = "^edge_.*",
+
+    def __init__(self, edge_indices: str = "edge_indices", edge_attributes: str = "^edge_(?!indices$).*",
                  remove_duplicates: bool = True, sort_indices: bool = True, name="make_undirected_edges", **kwargs):
         super().__init__(name=name, **kwargs)
         self._to_obtain.update({"edge_indices": edge_indices, "edge_attributes": edge_attributes})
@@ -37,8 +39,10 @@ class MakeUndirectedEdges(GraphPreProcessorBase):
                                     **self._call_kwargs})
 
     def call(self, edge_indices: np.ndarray, edge_attributes: list, remove_duplicates: bool, sort_indices: bool):
+        if edge_indices is None:
+            return None, [None] * len(edge_attributes)
         return add_edges_reverse_indices(
-            edge_indices=edge_indices, *edge_attributes, remove_duplicates=remove_duplicates, sort_indices=sort_indices,
+            edge_indices, *edge_attributes, remove_duplicates=remove_duplicates, sort_indices=sort_indices,
             return_nested=True)
 
 
@@ -55,7 +59,8 @@ class AddEdgeSelfLoops(GraphPreProcessorBase):
         sort_indices (bool): To sort indices after adding self-loops. Default is True.
         fill_value (in): The fill_value for all other edge properties.
     """
-    def __init__(self, edge_indices: str = "edge_indices", edge_attributes: str = "^edge_.*",
+
+    def __init__(self, edge_indices: str = "edge_indices", edge_attributes: str = "^edge_(?!indices$).*",
                  remove_duplicates: bool = True, sort_indices: bool = True, fill_value: int = 0,
                  name="add_edge_self_loops", **kwargs):
         super().__init__(name=name, **kwargs)
@@ -69,8 +74,10 @@ class AddEdgeSelfLoops(GraphPreProcessorBase):
 
     def call(self, *, edge_indices: np.ndarray, edge_attributes: list, remove_duplicates: bool, sort_indices: bool,
              fill_value: bool):
+        if edge_indices is None:
+            return None, [None] * len(edge_attributes)
         return add_self_loops_to_edge_indices(
-            edge_indices=edge_indices, *edge_attributes, remove_duplicates=remove_duplicates,
+            edge_indices, *edge_attributes, remove_duplicates=remove_duplicates,
             sort_indices=sort_indices, fill_value=fill_value, return_nested=True)
 
 
@@ -82,7 +89,8 @@ class SortEdgeIndices(GraphPreProcessorBase):
         edge_attributes (str): Name of related edge values or attributes.
             This can be a match-string or list of names. Default is "^edge_.*".
     """
-    def __init__(self, *, edge_indices: str = "edge_indices", edge_attributes: str = "^edge_.*",
+
+    def __init__(self, *, edge_indices: str = "edge_indices", edge_attributes: str = "^edge_(?!indices$).*",
                  name="sort_edge_indices", **kwargs):
         super().__init__(name=name, **kwargs)
         self._to_obtain.update({"edge_indices": edge_indices, "edge_attributes": edge_attributes})
@@ -91,6 +99,8 @@ class SortEdgeIndices(GraphPreProcessorBase):
         self._config_kwargs.update({"edge_indices": edge_indices, "edge_attributes": edge_attributes})
 
     def call(self, *, edge_indices: np.ndarray, edge_attributes: list):
+        if edge_indices is None:
+            return None, [None] * len(edge_attributes)
         return sort_edge_indices(edge_indices, *edge_attributes, return_nested=True)
 
 
@@ -107,6 +117,7 @@ class SetEdgeIndicesReverse(GraphPreProcessorBase):
         edge_indices (str): Name of indices in dictionary. Default is "edge_indices".
         edge_indices_reverse (str): Name of reverse indices to store output. Default is "edge_indices_reverse"
     """
+
     def __init__(self, *, edge_indices: str = "edge_indices", edge_indices_reverse: str = "edge_indices_reverse",
                  name="set_edge_indices_reverse", **kwargs):
         super().__init__(name=name, **kwargs)
@@ -128,8 +139,9 @@ class PadProperty(GraphPreProcessorBase):
         pad_width (list, int): Width to pad tensor.
         mode (str): Padding mode.
     """
+
     def __init__(self, *, key: str, pad_width: Union[int, list, tuple, np.ndarray], mode: str = "constant",
-                 name="name", **kwargs):
+                 name="pad_property", **kwargs):
         call_kwargs = {"pad_width": pad_width, "mode": mode}
         # List of additional kwargs for pad that do not belong to super.
         for x in ["stat_length", "constant_values", "end_values", "reflect_type"]:
@@ -188,7 +200,7 @@ class NormalizeEdgeWeightsSymmetric(GraphPreProcessorBase):
         super().__init__(name=name, **kwargs)
         self._to_obtain.update({"edge_indices": edge_indices, "edge_weights": edge_weights})
         self._to_assign = edge_weights
-        self._silent = edge_weights
+        self._silent = ["edge_weights"]
         self._config_kwargs.update({"edge_indices": edge_indices, "edge_weights": edge_weights})
 
     def call(self, *, edge_indices: np.ndarray, edge_weights: np.ndarray):
@@ -213,6 +225,7 @@ class SetRangeFromEdges(GraphPreProcessorBase):
         range_attributes (str): Name of range distance to set in dictionary. Default is "range_attributes".
         do_invert_distance (bool): Invert distance when computing  :obj:`range_attributes`. Default is False.
     """
+
     def __init__(self, *, edge_indices: str = "edge_indices", range_indices: str = "range_indices",
                  node_coordinates: str = "node_coordinates", range_attributes: str = "range_attributes",
                  do_invert_distance: bool = False, name="set_range_from_edges", **kwargs):
@@ -307,25 +320,34 @@ class SetAngle(GraphPreProcessorBase):
             the nodes are unique. Default is False.
         compute_angles (bool): Whether to also compute angles.
     """
+
     def __init__(self, *, range_indices: str = "range_indices", node_coordinates: str = "node_coordinates",
                  angle_indices: str = "angle_indices", angle_indices_nodes: str = "angle_indices_nodes",
-                 angle_attributes: str = "angle_attributes", allow_multi_edges: bool = False,
+                 angle_attributes: str = "angle_attributes",
+                 allow_multi_edges: bool = False, allow_self_edges: bool = False, allow_reverse_edges: bool = False,
+                 edge_pairing: str = "kj", check_sorted: bool = True,
                  compute_angles: bool = True, name="set_angle", **kwargs):
         super().__init__(name=name, **kwargs)
         self._to_obtain.update({"node_coordinates": node_coordinates, "range_indices": range_indices})
-        self._call_kwargs = {"allow_multi_edges": allow_multi_edges, "compute_angles": compute_angles}
+        self._call_kwargs = {"allow_multi_edges": allow_multi_edges, "compute_angles": compute_angles,
+                             "allow_self_edges": allow_self_edges, "edge_pairing": edge_pairing,
+                             "allow_reverse_edges": allow_reverse_edges, "check_sorted": check_sorted}
         self._to_assign = [angle_indices, angle_indices_nodes, angle_attributes]
-        self._silent = node_coordinates
+        self._silent = ["node_coordinates"]
         self._config_kwargs.update({
             "node_coordinates": node_coordinates, "range_indices": range_indices, "angle_indices": angle_indices,
             "angle_indices_nodes": angle_indices_nodes, "angle_attributes": angle_attributes, **self._call_kwargs})
 
     def call(self, *, range_indices: np.ndarray, node_coordinates: np.ndarray,
-             allow_multi_edges: bool, compute_angles: bool):
+             check_sorted: bool, allow_multi_edges: bool,
+             allow_self_edges: bool, allow_reverse_edges: bool,
+             edge_pairing: str, compute_angles: bool):
         if range_indices is None:
             return None, None, None
         # Compute Indices
-        _, a_triples, a_indices = get_angle_indices(range_indices, allow_multi_edges=allow_multi_edges)
+        _, a_triples, a_indices = get_angle_indices(
+            range_indices, allow_multi_edges=allow_multi_edges, allow_self_edges=allow_self_edges,
+            allow_reverse_edges=allow_reverse_edges, check_sorted=check_sorted, edge_pairing=edge_pairing)
         # Also compute angles
         if compute_angles:
             if node_coordinates is not None:
@@ -353,6 +375,7 @@ class SetRangePeriodic(GraphPreProcessorBase):
         do_invert_distance (bool): Whether to invert the distance. Default is False.
         self_loops (bool): If also self-interactions with distance 0 should be considered. Default is False.
     """
+
     def __init__(self, *, range_indices: str = "range_indices", node_coordinates: str = "node_coordinates",
                  graph_lattice: str = "graph_lattice", range_image: str = "range_image",
                  range_attributes: str = "range_attributes", max_distance: float = 4.0, max_neighbours: int = None,
@@ -362,7 +385,7 @@ class SetRangePeriodic(GraphPreProcessorBase):
         self._to_obtain.update({"node_coordinates": node_coordinates, "graph_lattice": graph_lattice})
         self._call_kwargs = {
             "max_distance": max_distance, "max_neighbours": max_neighbours, "exclusive": exclusive,
-            "do_invert_distance": do_invert_distance, "self_loops":self_loops}
+            "do_invert_distance": do_invert_distance, "self_loops": self_loops}
         self._to_assign = [range_indices, range_image, range_attributes]
         self._config_kwargs.update({
             "node_coordinates": node_coordinates, "range_indices": range_indices, "graph_lattice": graph_lattice,
@@ -388,5 +411,70 @@ class SetRangePeriodic(GraphPreProcessorBase):
         return indices, images, dist
 
 
-def get():
-    pass
+class ExpandDistanceGaussianBasis(GraphPreProcessorBase):
+    r"""Expand distance into Gaussian basis a features or attributes.
+
+    Args:
+        range_attributes (str): Name of distances in dictionary. Default is "range_attributes".
+        bins (int): Number of bins to sample distance from. Default is 20.
+        distance (value): Maximum distance to be captured by bins. Default is 4.0.
+        sigma (value): Sigma of the Gaussian function, determining the width/sharpness. Default is 0.4.
+        offset (float): Possible offset to center Gaussian. Default is 0.0.
+        axis (int): Axis to expand distance. Defaults to -1.
+        expand_dims (bool): Whether to expand dims. Default to True.
+    """
+
+    def __init__(self, *, range_attributes: str = "range_attributes",
+                 bins: int = 20, distance: float = 4.0, sigma: float = 0.4, offset: float = 0.0, axis: int = -1,
+                 expand_dims: bool = True,
+                 name="expand_distance_gaussian_basis", **kwargs):
+        super().__init__(name=name, **kwargs)
+        self._to_obtain.update({"range_attributes": range_attributes})
+        self._to_assign = range_attributes
+        self._call_kwargs = {"bins": bins, "distance": distance, "sigma": sigma, "offset": offset, "axis": axis,
+                             "expand_dims": expand_dims}
+        self._config_kwargs.update({"range_attributes": range_attributes, **self._call_kwargs})
+
+    def call(self, *, range_attributes: np.ndarray,
+             bins: int, distance: float, sigma: float, offset: float, axis: int, expand_dims: bool):
+        if range_attributes is None:
+            return None
+        return distance_to_gauss_basis(range_attributes, bins=bins, distance=distance, sigma=sigma, offset=offset,
+                                       axis=axis, expand_dims=expand_dims)
+
+
+class AtomicChargesRepresentation(GraphPreProcessorBase):
+    r"""Generate a node representation based on atomic charges.
+    Multiplies a one-hot representation of charges with multiple powers of the (scaled) charge itself.
+    This has been used by `EGNN <https://arxiv.org/abs/2102.09844>`_.
+
+    Args:
+        node_number (str): Name of atomic charges in dictionary. Default is "node_number".
+        node_attributes (str): Name of related attributes to assign output. Defaults to "node_attributes".
+        one_hot (list): List of possible charges. Default is [1, 6, 7, 8, 9].
+        charge_scale (float): Scale by which the charges are scaled for power. Default is 9.0.
+        charge_power (int): Maximum power up to which compute charge powers. Default is 2.
+    """
+
+    def __init__(self, *, node_number: str = "node_number", node_attributes: str = "node_attributes",
+                 name="atomic_charge_representation", one_hot: list = None, charge_scale: float = 9.0,
+                 charge_power: int = 2, **kwargs):
+        super().__init__(name=name, **kwargs)
+        if one_hot is None:
+            one_hot = [1, 6, 7, 8, 9]
+        self._to_obtain.update({"node_number": node_number})
+        self._to_assign = node_attributes
+        self._call_kwargs = {"one_hot": one_hot, "charge_scale": charge_scale, "charge_power": charge_power}
+        self._config_kwargs.update({"node_number": node_number, "node_attributes": node_attributes,
+                                    **self._call_kwargs})
+
+    def call(self, *, node_number: np.ndarray, one_hot: list, charge_scale: float, charge_power: int):
+        if node_number is None:
+            return None
+        if len(node_number.shape) <= 1:
+            node_number = np.expand_dims(node_number, axis=-1)
+        oh = np.array(node_number == np.array([one_hot], dtype="int"), dtype="float")  # (N, ohe)
+        charge_tensor = np.power(node_number/charge_scale, np.arange(charge_power + 1))   # (N, power)
+        atom_scalars = np.expand_dims(oh, axis=-1) * np.expand_dims(charge_tensor, axis=1)  # (N, ohe, power)
+        return atom_scalars.reshape((len(node_number), -1))
+
