@@ -406,7 +406,7 @@ class SphericalBasisLayer(GraphBaseLayer):
         self.bessel_n_zeros = spherical_bessel_jn_zeros(num_spherical, num_radial)
         self.bessel_norm = spherical_bessel_jn_normalization_prefactor(num_spherical, num_radial)
 
-        self.layer_gather_out = GatherNodesOutgoing(**self._kgcnn_info)
+        self.layer_gather_out = GatherNodesOutgoing()
 
     @tf.function
     def envelope(self, inputs):
@@ -430,7 +430,7 @@ class SphericalBasisLayer(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Expanded angle/distance basis. Shape is (batch, [K], #Radial * #Spherical)
         """
-        self.assert_ragged_input_rank(inputs)
+        inputs = self.assert_ragged_input_rank(inputs)
         edge, edge_part = inputs[0].values, inputs[0].row_splits
         angles, angle_part = inputs[1].values, inputs[1].row_splits
 
