@@ -5,6 +5,7 @@ from kgcnn.layers.base import GraphBaseLayer
 from kgcnn.layers.gather import GatherNodesSelection, GatherState
 from kgcnn.layers.modules import LazySubtract, LazyMultiply, LazyAdd
 from kgcnn.ops.axis import get_positive_axis
+
 ks = tf.keras
 
 
@@ -135,39 +136,58 @@ class EuclideanNorm(GraphBaseLayer):
     with :obj:`invert_norm` layer arguments.
     """
 
-    def __init__(self, axis: int = -1, keepdims: bool = False, invert_norm: bool = False, **kwargs):
+    def __init__(self, axis: int = -1, keepdims: bool = False, invert_norm: bool = False, add_eps: bool = False,
+                 no_nan: bool = True, square_norm: bool = False, **kwargs):
         """Initialize layer.
 
         Args:
             axis (int): Axis of coordinates. Defaults to -1.
             keepdims (bool): Whether to keep the axis for sum. Defaults to False.
             invert_norm (bool): Whether to invert the results. Defaults to False.
+            add_eps (bool): Whether to add epsilon before taking square root. Default is False.
+            no_nan (bool): Whether to remove NaNs on invert. Default is True.
         """
         super(EuclideanNorm, self).__init__(**kwargs)
         self.axis = axis
         self.keepdims = keepdims
         self.invert_norm = invert_norm
+        self.square_norm = square_norm
+        self.add_eps = add_eps
+        self.no_nan = no_nan
 
     def build(self, input_shape):
         """Build layer."""
         super(EuclideanNorm, self).build(input_shape)
+        self.axis = get_positive_axis(self.axis, len(input_shape))
 
     @staticmethod
-    def _compute_euclidean_norm(inputs, axis: int = -1, keepdims: bool = False, invert_norm: bool = False):
+    def _compute_euclidean_norm(inputs, axis: int = -1, keepdims: bool = False, invert_norm: bool = False,
+                                add_eps: bool = False, no_nan: bool = True, square_norm: bool = False):
         """Function to compute euclidean norm for inputs.
 
         Args:
             inputs (tf.Tensor, tf.RaggedTensor): Tensor input to compute norm for.
             axis (int): Axis of coordinates. Defaults to -1.
             keepdims (bool): Whether to keep the axis for sum. Defaults to False.
+            add_eps (bool): Whether to add epsilon before taking square root. Default is False.
+            square_norm (bool): Whether to square the results. Defaults to False.
             invert_norm (bool): Whether to invert the results. Defaults to False.
 
         Returns:
             tf.Tensor: Euclidean norm of inputs.
         """
-        out = tf.sqrt(tf.nn.relu(tf.reduce_sum(tf.square(inputs), axis=axis, keepdims=keepdims)))
+        out = tf.nn.relu(tf.reduce_sum(tf.square(inputs), axis=axis, keepdims=keepdims))
+        # Or just via tf.norm.
+        # out = tf.norm(inputs, ord='euclidean', axis=axis, keepdims=keepdims)
+        if add_eps:
+            out = out + ks.backend.epsilon()
+        if not square_norm:
+            out = tf.sqrt(out)
         if invert_norm:
-            out = tf.math.divide_no_nan(tf.constant(1, dtype=out.dtype), out)
+            if no_nan:
+                out = tf.math.divide_no_nan(tf.constant(1, dtype=out.dtype), out)
+            else:
+                out = 1 / out
         return out
 
     def call(self, inputs, **kwargs):
@@ -179,18 +199,16 @@ class EuclideanNorm(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Euclidean norm computed for specific axis of shape `(batch, [N], ...)`
         """
-        # Possibly faster
-        if isinstance(inputs, tf.RaggedTensor):
-            axis = get_positive_axis(self.axis, inputs.shape.rank)
-            if inputs.ragged_rank == 1 and axis > 1:
-                out = self._compute_euclidean_norm(inputs.values, axis-1, self.keepdims, self.invert_norm)
-                return tf.RaggedTensor.from_row_splits(out, inputs.row_splits, validate=self.ragged_validate)
-        return self._compute_euclidean_norm(inputs, self.axis, self.keepdims, self.invert_norm)
+        return self.call_on_values_tensor_of_ragged(
+            self._compute_euclidean_norm, inputs,
+            axis=self.axis, keepdims=self.keepdims, invert_norm=self.invert_norm, add_eps=self.add_eps,
+            no_nan=self.no_nan, square_norm=self.square_norm)
 
     def get_config(self):
         """Update config."""
         config = super(EuclideanNorm, self).get_config()
-        config.update({"axis": self.axis, "keepdims": self.keepdims, "invert_norm": self.invert_norm})
+        config.update({"axis": self.axis, "keepdims": self.keepdims, "invert_norm": self.invert_norm,
+                       "add_eps": self.add_eps, "no_nan": self.no_nan, "square_norm": self.square_norm})
         return config
 
 
@@ -221,6 +239,10 @@ class ScalarProduct(GraphBaseLayer):
     def build(self, input_shape):
         """Build layer."""
         super(ScalarProduct, self).build(input_shape)
+        axis = get_positive_axis(self.axis, len(input_shape[0]))
+        axis2 = get_positive_axis(self.axis, len(input_shape[1]))
+        assert axis2 == axis, "Axis parameter must match on the two input vectors for scalar product."
+        self.axis = axis
 
     @staticmethod
     def _scalar_product(inputs: list, axis: int, **kwargs):
@@ -247,17 +269,7 @@ class ScalarProduct(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Scalar product of shape `(batch, [N], ...)`
         """
-        if all([isinstance(x, tf.RaggedTensor) for x in inputs]):  # Possibly faster
-            axis = get_positive_axis(self.axis, inputs[0].shape.rank)
-            axis2 = get_positive_axis(self.axis, inputs[1].shape.rank)
-            assert axis2 == axis
-            if all([x.ragged_rank == 1 for x in inputs]) and axis > 1:
-                v1, v2 = inputs[0].values, inputs[1].values
-                out = self._scalar_product([v1, v2], axis=axis-1)
-                return tf.RaggedTensor.from_row_splits(out, inputs[0].row_splits, validate=self.ragged_validate)
-        # Default
-        out = self._scalar_product(inputs, axis=self.axis)
-        return out
+        return self.call_on_values_tensor_of_ragged(self._scalar_product, inputs, axis=self.axis)
 
     def get_config(self):
         """Update config."""
@@ -280,11 +292,11 @@ class NodeDistanceEuclidean(GraphBaseLayer):
 
     """
 
-    def __init__(self, **kwargs):
+    def __init__(self, add_eps: bool = False, no_nan: bool = True, **kwargs):
         r"""Initialize layer instance of :obj:`NodeDistanceEuclidean`. """
         super(NodeDistanceEuclidean, self).__init__(**kwargs)
         self.layer_subtract = LazySubtract()
-        self.layer_euclidean_norm = EuclideanNorm(axis=2, keepdims=True)
+        self.layer_euclidean_norm = EuclideanNorm(axis=2, keepdims=True, add_eps=add_eps, no_nan=no_nan)
 
     def build(self, input_shape):
         """Build layer."""
@@ -305,6 +317,12 @@ class NodeDistanceEuclidean(GraphBaseLayer):
         diff = self.layer_subtract(inputs)
         return self.layer_euclidean_norm(diff)
 
+    def get_config(self):
+        config = super(NodeDistanceEuclidean, self).get_config()
+        conf_norm = self.layer_euclidean_norm.get_config()
+        config.update({"add_eps": conf_norm["add_eps"], "no_nan": conf_norm["no_nan"]})
+        return config
+
 
 @ks.utils.register_keras_serializable(package='kgcnn', name='EdgeDirectionNormalized')
 class EdgeDirectionNormalized(GraphBaseLayer):
@@ -320,11 +338,12 @@ class EdgeDirectionNormalized(GraphBaseLayer):
     As the first index defines the incoming edge.
     """
 
-    def __init__(self, **kwargs):
+    def __init__(self, add_eps: bool = False, no_nan: bool = True, **kwargs):
         """Initialize layer."""
         super(EdgeDirectionNormalized, self).__init__(**kwargs)
         self.layer_subtract = LazySubtract()
-        self.layer_euclidean_norm = EuclideanNorm(axis=2, keepdims=True, invert_norm=True)
+        self.layer_euclidean_norm = EuclideanNorm(
+            axis=2, keepdims=True, invert_norm=True, add_eps=add_eps, no_nan=no_nan)
         self.layer_multiply = LazyMultiply()
 
     def build(self, input_shape):
@@ -350,6 +369,8 @@ class EdgeDirectionNormalized(GraphBaseLayer):
     def get_config(self):
         """Update config."""
         config = super(EdgeDirectionNormalized, self).get_config()
+        conf_norm = self.layer_euclidean_norm.get_config()
+        config.update({"add_eps": conf_norm["add_eps"], "no_nan": conf_norm["no_nan"]})
         return config
 
 
@@ -412,11 +433,7 @@ class VectorAngle(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Calculated Angle between vector 1 and 2 of shape `(batch, [M], 1)`.
         """
-        if all([isinstance(x, tf.RaggedTensor) for x in inputs]):  # Possibly faster
-            if all([x.ragged_rank == 1 for x in inputs]):
-                angle = self._compute_vector_angle([inputs[0].values, inputs[1].values])
-                return tf.RaggedTensor.from_row_splits(angle, inputs[0].row_splits, validate=self.ragged_validate)
-        return self._compute_vector_angle(inputs)
+        return self.call_on_values_tensor_of_ragged(self._compute_vector_angle, inputs)
 
     def get_config(self):
         """Update config."""
@@ -443,16 +460,24 @@ class EdgeAngle(GraphBaseLayer):
 
     """
 
-    def __init__(self, **kwargs):
+    def __init__(self, vector_scale: list = None, **kwargs):
         """Initialize layer."""
         super(EdgeAngle, self).__init__(**kwargs)
         self.layer_gather_vectors = GatherNodesSelection([0, 1])
         self.layer_angle = VectorAngle()
+        self.vector_scale = vector_scale
+        if vector_scale:
+            assert len(vector_scale) == 2, "Need scale for both vectors to compute angle."
+        self._tf_vec_scale = [tf.constant(x) for x in self.vector_scale] if self.vector_scale else None
 
     def build(self, input_shape):
         """Build layer."""
         super(EdgeAngle, self).build(input_shape)
 
+    @staticmethod
+    def _scale_vector(x, scale):
+        return x * tf.cast(scale, dtype=x.dtype)
+
     def call(self, inputs, **kwargs):
         r"""Forward pass.
 
@@ -466,11 +491,15 @@ class EdgeAngle(GraphBaseLayer):
             tf.RaggedTensor: Edge angles between edges that match the indices. Shape is `(batch, [K], 1)`.
         """
         v1, v2 = self.layer_gather_vectors(inputs)
+        if self.vector_scale:
+            v1, v2 = [self.call_on_values_tensor_of_ragged(self._scale_vector, x, scale=self._tf_vec_scale[i]) for i, x
+                      in enumerate([v1, v2])]
         return self.layer_angle([v1, v2])
 
     def get_config(self):
         """Update config."""
         config = super(EdgeAngle, self).get_config()
+        config.update({"vector_scale": self.vector_scale})
         return config
 
 
@@ -540,17 +569,14 @@ class GaussBasisLayer(GraphBaseLayer):
         Args:
             inputs: distance
 
-                - distance (tf.RaggedTensor): Edge distance of shape `(batch, [K], 1)`
+                - distance (tf.RaggedTensor, tf.Tensor): Edge distance of shape `(batch, [K], 1)`
 
         Returns:
             tf.RaggedTensor: Expanded distance. Shape is `(batch, [K], bins)`.
         """
-        # Possibly faster RaggedRank==1
-        if isinstance(inputs, tf.RaggedTensor):
-            if inputs.ragged_rank == 1:
-                out = self._compute_gauss_basis(inputs.values, self.offset, self.gamma, self.bins, self.distance)
-                return tf.RaggedTensor.from_row_splits(out, inputs.row_splits, validate=self.ragged_validate)
-        return self._compute_gauss_basis(inputs, self.offset, self.gamma, self.bins, self.distance)
+        return self.call_on_values_tensor_of_ragged(
+            self._compute_gauss_basis, inputs,
+            offset=self.offset, gamma=self.gamma, bins=self.bins, distance=self.distance)
 
     def get_config(self):
         """Update config."""
@@ -567,11 +593,7 @@ class PositionEncodingBasisLayer(GraphBaseLayer):
 
     """
 
-    def __init__(self,
-                 dim_half: int = 8,
-                 wave_length: float = 10,
-                 include_frequencies: bool = False,
-                 **kwargs):
+    def __init__(self, dim_half: int = 8, wave_length: float = 10, include_frequencies: bool = False, **kwargs):
         r"""Initialize :obj:`FourierBasisLayer` layer.
 
         Args:
@@ -602,7 +624,7 @@ class PositionEncodingBasisLayer(GraphBaseLayer):
             tf.Tensor: Distance tensor expanded in Fourier basis.
         """
         k = -math.log(wave_length)
-        steps = tf.range(dim_half, dtype=inputs.dtype)/dim_half
+        steps = tf.range(dim_half, dtype=inputs.dtype) / dim_half
         freq = tf.exp(k * steps)
         scales = tf.cast(freq, dtype=inputs.dtype) * 2 * math.pi
         arg = inputs * scales
@@ -617,13 +639,13 @@ class PositionEncodingBasisLayer(GraphBaseLayer):
         r"""Forward pass.
 
         Args:
-            inputs (tf.RaggedTensor): Edge distance of shape `(batch, [K], 1)`
+            inputs (tf.RaggedTensor, tf.Tensor): Edge distance of shape `(batch, [K], 1)`
 
         Returns:
             tf.RaggedTensor: Expanded distance. Shape is `(batch, [K], bins)`.
         """
         return self.call_on_values_tensor_of_ragged(
-            self._compute_gauss_basis, inputs,
+            self._compute_fourier_encoding, inputs,
             dim_half=self.dim_half, wave_length=self.wave_length, include_frequencies=self.include_frequencies)
 
     def get_config(self):
@@ -655,7 +677,6 @@ class BesselBasisLayer(GraphBaseLayer):
         u(d) = 1 − \frac{(p + 1)(p + 2)}{2} d^p + p(p + 2)d^{p+1} − \frac{p(p + 1)}{2} d^{p+2},
 
     where :math:`p \in \mathbb{N}_0` and typically :math:`p=6`.
-
     """
 
     def __init__(self, num_radial: int,
@@ -715,12 +736,7 @@ class BesselBasisLayer(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Expanded distance. Shape is `(batch, [K], num_radial)`.
         """
-        # Possibly faster RaggedRank==1
-        if isinstance(inputs, tf.RaggedTensor):
-            if inputs.ragged_rank == 1:
-                out = self.expand_bessel_basis(inputs.values)
-                return tf.RaggedTensor.from_row_splits(out, inputs.row_splits, validate=self.ragged_validate)
-        return self.expand_bessel_basis(inputs)
+        return self.call_on_values_tensor_of_ragged(self.expand_bessel_basis, inputs)
 
     def get_config(self):
         """Update config."""
@@ -741,7 +757,6 @@ class CosCutOffEnvelope(GraphBaseLayer):
         f_c(R_{ij}) = 0.5 [\cos{\frac{\pi R_{ij}}{R_c}} + 1]
 
     This layer only computes the cutoff envelope but does not apply it.
-
     """
 
     def __init__(self,
@@ -794,12 +809,9 @@ class CosCutOff(GraphBaseLayer):
         f_c(R_{ij}) = 0.5 [\cos{\frac{\pi R_{ij}}{R_c}} + 1]
 
     This layer computes the cutoff envelope and applies it to the input by simply multiplying with the envelope.
-
     """
 
-    def __init__(self,
-                 cutoff,
-                 **kwargs):
+    def __init__(self, cutoff, **kwargs):
         r"""Initialize layer.
 
         Args:
@@ -874,17 +886,17 @@ class DisplacementVectorsASU(GraphBaseLayer):
         in_frac_coords, out_frac_coords = self.gather_node_positions([frac_coords, edge_indices], **kwargs)
         in_frac_coords = in_frac_coords.values
         out_frac_coords = out_frac_coords.values
-        
+
         # Affine Transformation
         out_frac_coords_ = tf.concat(
             [out_frac_coords, tf.expand_dims(tf.ones_like(out_frac_coords[:, 0]), axis=1)], axis=1)
         affine_matrices = symmops
         out_frac_coords = tf.einsum('ij,ikj->ik', out_frac_coords_, affine_matrices)[:, :-1]
         out_frac_coords = out_frac_coords - tf.floor(out_frac_coords)  # All values should be in [0,1) interval
-        
+
         # Cell translation
         out_frac_coords = out_frac_coords + cell_translations
-        
+
         offset = in_frac_coords - out_frac_coords
         return tf.RaggedTensor.from_row_splits(offset, edge_indices.row_splits, validate=self.ragged_validate)
 
@@ -925,7 +937,7 @@ class DisplacementVectorsUnitCell(GraphBaseLayer):
 
         # Gather sending and receiving coordinates.
         in_frac_coords, out_frac_coords = self.gather_node_positions([frac_coords, edge_indices], **kwargs)
-        
+
         # Cell translation
         out_frac_coords = self.lazy_add([out_frac_coords, cell_translations], **kwargs)
         offset = self.lazy_sub([in_frac_coords, out_frac_coords], **kwargs)
@@ -962,5 +974,5 @@ class FracToRealCoordinates(GraphBaseLayer):
         frac_coords = inputs[0]
         lattice_matrices = inputs[1]
         lattice_matrices_ = tf.repeat(lattice_matrices, frac_coords.row_lengths(), axis=0)
-        real_coords = tf.einsum('ij,ikj->ik', frac_coords.values,  lattice_matrices_)
+        real_coords = tf.einsum('ij,ikj->ik', frac_coords.values, lattice_matrices_)
         return tf.RaggedTensor.from_row_splits(real_coords, frac_coords.row_splits, validate=self.ragged_validate)
